@@ -9,5 +9,5 @@ from .log import CanutilsLogReader, CanutilsLogWriter
 from .asc import ASCWriter, ASCReader
 from .blf import BLFReader, BLFWriter
 from .csv import CSVWriter
-from .sqlite import SqlReader, SqliteWriter
+from .sqlite import SqliteReader, SqliteWriter
 from .stdout import Printer
