@@ -1,61 +1,89 @@
+from datetime import datetime
+import time
+import logging
+
 from can.listener import Listener
 from can.message import Message
 
-from datetime import datetime
-import time
 CAN_MSG_EXT = 0x80000000
 CAN_ID_MASK = 0x1FFFFFFF
 
+logger = logging.getLogger('can.io.asc')
+
+
 class ASCReader(object):
     """
     Iterator of CAN messages from a ASC Logging File.
     """
 
     def __init__(self, filename):
-        self.fp = open(filename, "r")
+        self.file = open(filename, 'r')
+
+    @staticmethod
+    def _extract_can_id(str_can_id):
+        if str_can_id[-1:].lower() == "x":
+            is_extended = True
+            can_id = int(str_can_id[0:-1], 16)
+        else:
+            is_extended = False
+            can_id = int(str_can_id, 16)
+        logging.debug('ASCReader: _extract_can_id("%s") -> %x, %r', str_can_id, can_id, is_extended)
+        return (can_id, is_extended)
 
     def __iter__(self):
-        def extractCanId(strCanId):
-            if strCanId[-1:].lower() == "x":
-                isExtended = True
-                can_id = int(strCanId[0:-1], 16)
-            else:
-                isExtended = False
-                can_id = int(strCanId, 16)
-            return (can_id, isExtended)
+        for line in self.file:
+            logger.debug("ASCReader: parsing line: '%s'", line.splitlines()[0])
 
-        for line in self.fp:
             temp = line.strip()
-            if len(temp) == 0 or not temp[0].isdigit():
+            if not temp or not temp[0].isdigit():
                 continue
-            (time, channel, dummy) =  temp.split(None,2) # , frameType, dlc, frameData
 
-            time = float(time)
-            if dummy.strip()[0:10] == "ErrorFrame":
-                time = float(time)
-                msg = Message(timestamp=time, is_error_frame=True)
+            try:
+                (timestamp, channel, dummy) = temp.split(None, 2) # , frameType, dlc, frameData
+            except ValueError:
+                # we parsed an empty comment
+                continue
+
+            timestamp = float(timestamp)
+
+            if dummy.strip()[0:10] == 'ErrorFrame':
+                msg = Message(timestamp=timestamp, is_error_frame=True)
                 yield msg
-                continue
-            if not channel.isdigit() or dummy.strip()[0:10] == "Statistic:":
-                continue
-            if dummy[-1:].lower() == "r":
-                (canId, _) = dummy.split(None, 1)
-                msg = Message(timestamp=time,
-                              arbitration_id=extractCanId(canId)[0] & CAN_ID_MASK,
-                              extended_id=extractCanId(canId)[1],
+
+            elif not channel.isdigit() or dummy.strip()[0:10] == 'Statistic:':
+                pass
+
+            elif dummy[-1:].lower() == 'r':
+                (can_id_str, _) = dummy.split(None, 1)
+                (can_id_num, is_extended_id) = self._extract_can_id(can_id_str)
+                msg = Message(timestamp=timestamp,
+                              arbitration_id=can_id_num & CAN_ID_MASK,
+                              extended_id=is_extended_id,
                               is_remote_frame=True)
                 yield msg
+
             else:
-                (canId, direction,_,dlc,data) = dummy.split(None,4)
+                try:
+                    # this only works if dlc > 0 and thus data is availabe
+                    (can_id_str, _, _, dlc, data) = dummy.split(None, 4)
+                except ValueError:
+                    # but if not, we only want to get the stuff up to the dlc
+                    (can_id_str, _, _, dlc      ) = dummy.split(None, 3)
+                    # and we set data to an empty sequence manually
+                    data = ''
 
                 dlc = int(dlc)
                 frame = bytearray()
                 data = data.split()
                 for byte in data[0:dlc]:
-                    frame.append(int(byte,16))
-                msg = Message(timestamp=time,
-                            arbitration_id=extractCanId(canId)[0] & CAN_ID_MASK,
-                            extended_id=extractCanId(canId)[1],
+                    frame.append(int(byte, 16))
+
+                (can_id_num, is_extended_id) = self._extract_can_id(can_id_str)
+
+                msg = Message(
+                            timestamp=timestamp,
+                            arbitration_id=can_id_num & CAN_ID_MASK,
+                            extended_id=is_extended_id,
                             is_remote_frame=False,
                             dlc=dlc,
                             data=frame)
@@ -65,14 +93,14 @@ class ASCReader(object):
 class ASCWriter(Listener):
     """Logs CAN data to an ASCII log file (.asc)"""
 
-    LOG_STRING = "{time: 9.4f} {channel}  {id:<15} Rx   {dtype} {data}\n"
-    EVENT_STRING = "{time: 9.4f} {message}\n"
+    LOG_STRING      = "{time: 9.4f} {channel}  {id:<15} Rx   {dtype} {data}\n"
+    EVENT_STRING    = "{time: 9.4f} {message}\n"
 
     def __init__(self, filename, channel=1):
         now = datetime.now().strftime("%a %b %m %I:%M:%S %p %Y")
         self.channel = channel
         self.started = time.time()
-        self.log_file = open(filename, "w")
+        self.log_file = open(filename, 'w')
         self.log_file.write("date %s\n" % now)
         self.log_file.write("base hex  timestamps absolute\n")
         self.log_file.write("internal events logged\n")
@@ -87,6 +115,11 @@ class ASCWriter(Listener):
 
     def log_event(self, message, timestamp=None):
         """Add an arbitrary message to the log file."""
+
+        if not message: # if empty or None
+            logger.debug("ASCWriter: ignoring empty message")
+            return
+
         timestamp = (timestamp or time.time())
         if timestamp >= self.started:
             timestamp -= self.started
@@ -97,7 +130,7 @@ class ASCWriter(Listener):
 
     def on_message_received(self, msg):
         if msg.is_error_frame:
-            self.log_event("{} ErrorFrame".format(self.channel), msg.timestamp)
+            self.log_event("{}  ErrorFrame".format(self.channel), msg.timestamp)
             return
 
         if msg.is_remote_frame:
