@@ -1,11 +1,17 @@
-from can.listener import BufferedReader
-from can.message import Message
+"""
+Implements an SQL database writer and reader for storing CAN messages.
+
+The database schema is given in the documentation of the loggers.
+"""
 
 import sys
 import time
 import threading
-import sqlite3
 import logging
+import sqlite3
+
+from can.listener import BufferedReader
+from can.message import Message
 
 log = logging.getLogger('can.io.sql')
 
@@ -13,25 +19,52 @@ if sys.version_info > (3,):
     buffer = memoryview
 
 
-class SqlReader:
+class SqliteReader:
+    """
+    Reads recorded CAN messages from a simple SQL database.
+
+    This class can be iterated over or used to fetch all messages in the
+    database with :meth:`~SqliteReader.read_all`.
+
+    Calling len() on this object might not run in constant time.
+    """
+
+    _SELECT_ALL_COMMAND = "SELECT * FROM messages"
+
     def __init__(self, filename):
-        log.debug("Starting sqlreader with {}".format(filename))
-        conn = sqlite3.connect(filename)
-
-        self.c = conn.cursor()
-
+        log.debug("Starting SqliteReader with %s", filename)
+        self.conn = sqlite3.connect(filename)
+        self.cursor = self.conn.cursor()
 
     @staticmethod
-    def create_frame_from_db_tuple(frame_data):
-        ts, id, is_extended, is_remote, is_error, dlc, data = frame_data
+    def _create_frame_from_db_tuple(frame_data):
+        timestamp, can_id, is_extended, is_remote, is_error, dlc, data = frame_data
         return Message(
-            ts, is_remote, is_extended, is_error, id, dlc, data
+            timestamp, is_remote, is_extended, is_error, can_id, dlc, data
         )
 
     def __iter__(self):
         log.debug("Iterating through messages from sql db")
-        for frame_data in self.c.execute("SELECT * FROM messages"):
-            yield SqlReader.create_frame_from_db_tuple(frame_data)
+        for frame_data in self.cursor.execute(self._SELECT_ALL_COMMAND):
+            yield SqliteReader._create_frame_from_db_tuple(frame_data)
+
+    def __len__(self):
+        # this might not run in constant time
+        result = self.cursor.execute("SELECT COUNT(*) FROM messages")
+        return abs(int(result.fetchone()[0]))
+
+    def read_all(self):
+        """Fetches all messages in the database."""
+        result = self.cursor.execute(self._SELECT_ALL_COMMAND)
+        return result.fetchall()
+
+    def close(self):
+        """Closes the connection to the database."""
+        self.conn.close()
+
+
+# Backward compatibility
+SqlReader = SqliteReader
 
 
 class SqliteWriter(BufferedReader):
@@ -59,10 +92,10 @@ class SqliteWriter(BufferedReader):
 
     """
 
-    insert_msg_template = '''
+    _INSERT_MSG_TEMPLATE = '''
         INSERT INTO messages VALUES
         (?, ?, ?, ?, ?, ?, ?)
-        '''
+    '''
 
     GET_MESSAGE_TIMEOUT = 0.25
     """Number of seconds to wait for messages from internal queue"""
@@ -74,18 +107,18 @@ class SqliteWriter(BufferedReader):
         super(SqliteWriter, self).__init__()
         self.db_fn = filename
         self.stop_running_event = threading.Event()
-        self.writer_thread = threading.Thread(target=self.db_writer_thread)
+        self.writer_thread = threading.Thread(target=self._db_writer_thread)
         self.writer_thread.start()
 
     def _create_db(self):
-        # Note you can't share sqlite3 connections between threads
+        # Note: you can't share sqlite3 connections between threads
         # hence we setup the db here.
-        log.info("Creating sqlite db")
+        log.info("Creating sqlite database")
         self.conn = sqlite3.connect(self.db_fn)
-        c = self.conn.cursor()
+        cursor = self.conn.cursor()
 
         # create table structure
-        c.execute('''
+        cursor.execute('''
         CREATE TABLE IF NOT EXISTS messages
         (
           ts REAL,
@@ -99,9 +132,7 @@ class SqliteWriter(BufferedReader):
         ''')
         self.conn.commit()
 
-        self.db_setup = True
-
-    def db_writer_thread(self):
+    def _db_writer_thread(self):
         num_frames = 0
         last_write = time.time()
         self._create_db()
@@ -109,33 +140,37 @@ class SqliteWriter(BufferedReader):
         while not self.stop_running_event.is_set():
             messages = []
 
-            m = self.get_message(self.GET_MESSAGE_TIMEOUT)
-            while m is not None:
-                log.debug("sqlitewriter buffering message")
+            msg = self.get_message(self.GET_MESSAGE_TIMEOUT)
+            while msg is not None:
+                log.debug("SqliteWriter: buffering message")
 
                 messages.append((
-                    m.timestamp,
-                    m.arbitration_id,
-                    m.id_type,
-                    m.is_remote_frame,
-                    m.is_error_frame,
-                    m.dlc,
-                    buffer(m.data)
+                    msg.timestamp,
+                    msg.arbitration_id,
+                    msg.id_type,
+                    msg.is_remote_frame,
+                    msg.is_error_frame,
+                    msg.dlc,
+                    buffer(msg.data)
                 ))
 
                 if time.time() - last_write > self.MAX_TIME_BETWEEN_WRITES:
                     log.debug("Max timeout between writes reached")
                     break
 
-                m = self.get_message(self.GET_MESSAGE_TIMEOUT)
+                msg = self.get_message(self.GET_MESSAGE_TIMEOUT)
 
-            if len(messages) > 0:
+            count = len(messages)
+            if count > 0:
                 with self.conn:
-                    log.debug("Writing %s frames to db", len(messages))
-                    self.conn.executemany(SqliteWriter.insert_msg_template, messages)
-                    num_frames += len(messages)
+                    log.debug("Writing %s frames to db", count)
+                    self.conn.executemany(SqliteWriter._INSERT_MSG_TEMPLATE, messages)
+                    self.conn.commit() # make the changes visible to the entire database
+                    num_frames += count
                     last_write = time.time()
 
+            # go back up and check if we are still supposed to run
+
         self.conn.close()
         log.info("Stopped sqlite writer after writing %s messages", num_frames)
 
@@ -143,4 +178,3 @@ class SqliteWriter(BufferedReader):
         self.stop_running_event.set()
         log.debug("Stopping sqlite writer")
         self.writer_thread.join()
-
